@@ -1,6 +1,6 @@
 /*
- * Copyright (C) 2006, Mr Jamie McCracken (jamiemcc@gnome.org)
- * Copyright (C) 2009, Nokia
+ * Copyright (C) 2006, Jamie McCracken <jamiemcc@gnome.org>
+ * Copyright (C) 2009, Nokia <ivan.frade@nokia.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -84,13 +84,21 @@ typedef struct {
 	gchar *city;
 } TrackerXmpData;
 
-gboolean tracker_xmp_read  (const gchar          *buffer,
-                            size_t                len,
-                            const gchar          *uri,
-                            TrackerXmpData       *data);
-gboolean tracker_xmp_apply (TrackerSparqlBuilder *metadata,
-                            const gchar          *uri,
-                            TrackerXmpData       *data);
+TrackerXmpData * tracker_xmp_new   (const gchar          *buffer,
+                                    gsize                 len,
+                                    const gchar          *uri);
+void             tracker_xmp_free  (TrackerXmpData       *data);
+gboolean         tracker_xmp_apply (TrackerSparqlBuilder *metadata,
+                                    const gchar          *uri,
+                                    TrackerXmpData       *data);
+
+#ifndef TRACKER_DISABLE_DEPRECATED
+
+gboolean         tracker_xmp_read  (const gchar          *buffer,
+                                    size_t                len,
+                                    const gchar          *uri,
+                                    TrackerXmpData       *data) G_GNUC_DEPRECATED;
+#endif /* TRACKER_DISABLE_DEPRECATED */
 
 G_END_DECLS
 
