@@ -42,7 +42,7 @@
 #include <libtracker-common/tracker-type-utils.h>
 #include <libtracker-common/tracker-utils.h>
 
-#include <libtracker-db/tracker-db.h>
+#include <libtracker-data/tracker-data.h>
 
 #include "tracker-miner-files.h"
 #include "tracker-config.h"
@@ -58,6 +58,9 @@
  */
 #define N_DAYS_THRESHOLD 3
 
+/* Default DBus timeout to be used in requests to extractor (milliseconds) */
+#define EXTRACTOR_DBUS_TIMEOUT 60000
+
 #define TRACKER_MINER_FILES_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_MINER_FILES, TrackerMinerFilesPrivate))
 
 static GQuark miner_files_error_quark = 0;
@@ -606,6 +609,50 @@ miner_files_constructed (GObject *object)
 	disk_space_check_start (mf);
 }
 
+static void
+ensure_mount_point_exists (TrackerMinerFiles *miner,
+                           GFile             *mount_point,
+                           GString           *accumulator)
+{
+	gchar *iri;
+	gchar *uri;
+
+	uri = g_file_get_uri (mount_point);
+
+	/* Query the store for the URN of the mount point */
+	iri = tracker_miner_fs_query_urn (TRACKER_MINER_FS (miner),
+	                                  mount_point);
+
+	if (iri) {
+		/* If exists, just return, nothing else to do */
+		g_message ("Mount point '%s' already exists in store: '%s'",
+		           uri, iri);
+		g_free (iri);
+	} else {
+		/* If it doesn't exist, we need to create it */
+		g_message ("Mount point '%s' does not exist in store, need to create it",
+		           uri);
+
+		/* Create a nfo:Folder for the mount point */
+		g_string_append_printf (accumulator,
+		                        "INSERT SILENT INTO <" TRACKER_MINER_FS_GRAPH_URN "> {"
+		                        " _:file a nfo:FileDataObject, nie:InformationElement, nfo:Folder ; "
+		                        "        nie:isStoredAs _:file ; "
+		                        "        nie:url \"%s\" ; "
+		                        "        nie:mimeType \"inode/directory\" ; "
+		                        "        nfo:fileLastModified \"1981-06-05T02:20:00Z\" . "
+		                        "}",
+		                        uri);
+
+		/* Tell the underlying miner-fs that we created a directory without
+		 * a valid specific parent */
+		tracker_miner_fs_add_directory_without_parent (TRACKER_MINER_FS (miner),
+		                                               mount_point);
+	}
+
+	g_free (uri);
+}
+
 static void
 set_up_mount_point_cb (GObject      *source,
                        GAsyncResult *result,
@@ -668,12 +715,13 @@ set_up_mount_point (TrackerMinerFiles *miner,
 {
 	GString *queries;
 
-	g_debug ("Mount point state being set in DB for URN '%s'",
-	         removable_device_urn);
-
 	queries = g_string_new (NULL);
 
 	if (mounted) {
+		g_debug ("Mount point state (MOUNTED) being set in DB for URN '%s' (mount_point: %s)",
+		         removable_device_urn,
+		         mount_point ? mount_point : "unknown");
+
 		if (mount_point) {
 			GFile *file;
 			gchar *uri;
@@ -681,6 +729,10 @@ set_up_mount_point (TrackerMinerFiles *miner,
 			file = g_file_new_for_path (mount_point);
 			uri = g_file_get_uri (file);
 
+			/* Before assigning a nfo:FileDataObject as tracker:mountPoint for
+			 * the volume, make sure the nfo:FileDataObject exists in the store */
+			ensure_mount_point_exists (miner, file, queries);
+
 			g_string_append_printf (queries,
 			                        "DELETE FROM <%s> { "
 			                        "  <%s> tracker:mountPoint ?u "
@@ -719,6 +771,9 @@ set_up_mount_point (TrackerMinerFiles *miner,
 	} else {
 		gchar *now;
 
+		g_debug ("Mount point state (UNMOUNTED) being set in DB for URN '%s'",
+		         removable_device_urn);
+
 		now = tracker_date_to_string (time (NULL));
 
 		g_string_append_printf (queries,
@@ -781,11 +836,9 @@ init_mount_points_cb (GObject      *source,
 }
 
 static void
-query_mount_points_cb (GObject      *source,
-                       GAsyncResult *result,
-                       gpointer      user_data)
+init_mount_points (TrackerMinerFiles *miner_files)
 {
-	TrackerMiner *miner = TRACKER_MINER (source);
+	TrackerMiner *miner = TRACKER_MINER (miner_files);
 	TrackerMinerFilesPrivate *priv;
 	GHashTable *volumes;
 	GHashTableIter iter;
@@ -795,12 +848,17 @@ query_mount_points_cb (GObject      *source,
 	TrackerResultIterator *iterator;
 	GSList *uuids, *u;
 
-	iterator = tracker_miner_execute_sparql_finish (miner,
-	                                                result,
-	                                                &error);
+	g_debug ("Initializing mount points...");
+
+	/* First, get all mounted volumes, according to tracker-store (SYNC!) */
+	iterator = tracker_miner_execute_sparql_sync (TRACKER_MINER (miner),
+	                                              "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
+	                                              &error);
 	if (error) {
 		g_critical ("Could not obtain the mounted volumes: %s", error->message);
 		g_error_free (error);
+		if (iterator)
+			tracker_result_iterator_free (iterator);
 		return;
 	}
 
@@ -833,6 +891,8 @@ query_mount_points_cb (GObject      *source,
 		g_hash_table_replace (volumes, g_strdup (urn), GINT_TO_POINTER (state));
 	}
 
+	tracker_result_iterator_free (iterator);
+
 	/* Then, get all currently mounted non-REMOVABLE volumes, according to GIO */
 	uuids = tracker_storage_get_device_uuids (priv->storage, 0, TRUE);
 	for (u = uuids; u; u = u->next) {
@@ -949,20 +1009,6 @@ query_mount_points_cb (GObject      *source,
 	g_hash_table_unref (volumes);
 }
 
-static void
-init_mount_points (TrackerMinerFiles *miner)
-{
-	g_debug ("Initializing mount points");
-
-	/* First, get all mounted volumes, according to tracker-store */
-	tracker_miner_execute_sparql (TRACKER_MINER (miner),
-	                              "SELECT ?v WHERE { ?v a tracker:Volume ; tracker:isMounted true }",
-	                              NULL,
-	                              query_mount_points_cb,
-	                              NULL);
-}
-
-
 static gboolean
 cleanup_stale_removable_volumes_cb (gpointer user_data)
 {
@@ -1811,6 +1857,16 @@ extractor_create_proxy (DBusGConnection *connection)
 
 	if (!proxy) {
 		g_critical ("Could not create a DBusGProxy to the extract service");
+	} else {
+		/* Set default timeout for DBus requests to be around 60s.
+		 * Assuming that the files which need more time to get extracted are PDFs
+		 * using libpoppler, we already have a limit in the PDF extractor not to
+		 * spend more than 5s extraction contents. And, assuming the default
+		 * value of 10 in process-pool-limit, it means we may end up queueing up
+		 * to 10 PDF files which may need 5s each, so in order not to have dbus
+		 * timeouts in this case, any value greater than 5*10 would be good.
+		 */
+		dbus_g_proxy_set_default_timeout (proxy, EXTRACTOR_DBUS_TIMEOUT);
 	}
 
 	return proxy;
@@ -1830,6 +1886,8 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 		/* Something bad happened, notify about the error */
 		tracker_miner_fs_file_notify (TRACKER_MINER_FS (data->miner), data->file, error);
 		process_file_data_free (data);
+		/* Always free input GError. We want to behave exactly as if this
+		 * callback were one used in an async dbus-glib query.  */
 		g_error_free (error);
 		return;
 	}
@@ -1870,8 +1928,8 @@ extractor_get_embedded_metadata_cb (DBusGProxy *proxy,
 
 		removable_device_urn = g_strdup_printf (TRACKER_DATASOURCE_URN_PREFIX "%s", uuid);
 		uri = g_file_get_uri (G_FILE (data->file));
-
 		queries = g_string_new ("");
+
 		g_string_append_printf (queries,
 		                        "DELETE FROM <%s> { "
 		                        "  <%s> tracker:mountPoint ?unknown "
@@ -1966,6 +2024,9 @@ get_metadata_fast_cb (void     *buffer,
 		if (error->code != G_IO_ERROR_CANCELLED) {
 			/* ProcessFileData and error are freed in the callback */
 			(* data->callback) (NULL, NULL, NULL, error, process_data);
+		} else {
+			/* Free error ourselves */
+			g_error_free (error);
 		}
 	} else {
 		if (buffer_size) {
