@@ -163,6 +163,7 @@ typedef struct {
 	gboolean preserve_attribute_present;
 	const gchar *uri;
 	GString *content;
+	gboolean title_already_set;
 } MsOfficeXMLParserInfo;
 
 typedef struct {
@@ -419,7 +420,7 @@ msoffice_convert_and_normalize_chunk (guint8    *buffer,
 	 *
 	 * TODO: Using g_iconv, this extra heap allocation could be
 	 * avoided, re-using over and over again the same output buffer
-	 * for the UTF-8 encoded string 
+	 * for the UTF-8 encoded string
 	 */
 	converted_text = g_convert (buffer,
 	                            chunk_size,
@@ -444,7 +445,7 @@ msoffice_convert_and_normalize_chunk (guint8    *buffer,
 
 		/* Update accumulated UTF-8 bytes read */
 		*bytes_remaining -= len_to_validate;
-
+		g_free (converted_text);
 	} else {
 		g_warning ("Couldn't convert %" G_GSIZE_FORMAT " bytes from %s to UTF-8: %s",
 		           chunk_size,
@@ -846,6 +847,14 @@ extract_msword_content (GsfInfile *infile,
 	gsf_input_read (document_stream, 4, tmp_buffer);
 	lcbClx = read_32bit (tmp_buffer);
 
+	/* If we got an invalid or empty length of piece table, just return
+	 * as we cannot iterate over pieces */
+	if (lcbClx <= 0) {
+		g_object_unref (document_stream);
+		g_object_unref (table_stream);
+		return NULL;
+	}
+
 	/* copy the structure holding the piece table into the clx array. */
 	clx = g_malloc (lcbClx);
 	gsf_input_seek (table_stream, fcClx, G_SEEK_SET);
@@ -1649,6 +1658,8 @@ extract_msoffice (const gchar          *uri,
 
 	infile = open_uri (uri);
 	if (!infile) {
+		g_object_unref (file_info);
+		gsf_shutdown ();
 		return;
 	}
 
@@ -1686,6 +1697,7 @@ extract_msoffice (const gchar          *uri,
 	}
 
 	g_object_unref (infile);
+	g_object_unref (file_info);
 	gsf_shutdown ();
 }
 
@@ -1907,8 +1919,14 @@ xml_text_handler_document_data (GMarkupParseContext  *context,
 		break;
 
 	case MS_OFFICE_XML_TAG_TITLE:
-		tracker_sparql_builder_predicate (info->metadata, "nie:title");
-		tracker_sparql_builder_object_unvalidated (info->metadata, text);
+		if (info->title_already_set) {
+			g_warning ("Avoiding additional title (%s) in MsOffice XML document '%s'",
+			           text, info->uri);
+		} else {
+			info->title_already_set = TRUE;
+			tracker_sparql_builder_predicate (info->metadata, "nie:title");
+			tracker_sparql_builder_object_unvalidated (info->metadata, text);
+		}
 		break;
 
 	case MS_OFFICE_XML_TAG_SUBJECT:
@@ -2019,6 +2037,7 @@ xml_read (MsOfficeXMLParserInfo *parser_info,
 	info.preserve_attribute_present = FALSE;
 	info.uri = parser_info->uri;
 	info.content = parser_info->content;
+	info.title_already_set = parser_info->title_already_set;
 
 	switch (type) {
 	case MS_OFFICE_XML_TAG_DOCUMENT_CORE_DATA: {
@@ -2101,6 +2120,14 @@ xml_start_element_handler_content_types (GMarkupParseContext  *context,
 		}
 	}
 
+	/* Both part_name and content_type MUST be NON-NULL */
+	if (!part_name || !content_type) {
+		g_message ("Invalid file (part_name:%s, content_type:%s)",
+		           part_name ? part_name : "none",
+		           content_type ? content_type : "none");
+		return;
+	}
+
 	if ((g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-package.core-properties+xml") == 0) ||
 	    (g_ascii_strcasecmp (content_type, "application/vnd.openxmlformats-officedocument.extended-properties+xml") == 0)) {
 		xml_read (info, part_name + 1, MS_OFFICE_XML_TAG_DOCUMENT_CORE_DATA);
@@ -2202,6 +2229,7 @@ extract_msoffice_xml (const gchar          *uri,
 	info.preserve_attribute_present = FALSE;
 	info.uri = uri;
 	info.content = g_string_new ("");
+	info.title_already_set = FALSE;
 
 	context = g_markup_parse_context_new (&parser, 0, &info, NULL);
 
