@@ -737,6 +737,8 @@ context_menu_about_cb (GtkMenuItem *item,
 		"Mikael Ottela <mikael.ottela at ixonos com>",
 		"Ivan Frade <ivan.frade at nokia com>",
 		"Jamie McCracken <jamiemcc at gnome org>",
+		"Adrien Bustany <abustany at gnome org>",
+		"Aleksander Morgado <aleksander at lanedo com>",
 		"Anders Aagaard <aagaande at gmail com>",
 		"Anders Rune Jensen <anders iola dk>",
 		"Baptiste Mille-Mathias <baptist millemathias gmail com>",
@@ -801,7 +803,7 @@ context_menu_about_cb (GtkMenuItem *item,
 	gtk_show_about_dialog (NULL,
 	                       "version", PACKAGE_VERSION,
 	                       "comments", _("A notification icon application to monitor data miners for Tracker"),
-	                       "copyright", _("Copyright Tracker Authors 2005-2009"),
+	                       "copyright", _("Copyright Tracker Authors 2005-2010"),
 	                       "license", license_trans,
 	                       "wrap-license", TRUE,
 	                       "authors", authors,
@@ -834,12 +836,15 @@ status_icon_initialize_miners_menu (TrackerStatusIcon *icon)
 		image = gtk_image_new_from_icon_name (GTK_STOCK_FIND,
 						      GTK_ICON_SIZE_MENU);
 		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+		gtk_widget_show_all (item);
+
 		gtk_menu_shell_append (GTK_MENU_SHELL (priv->miner_menu), item);
 		g_signal_connect (G_OBJECT (item), "activate",
 				  G_CALLBACK (context_menu_search_cb), icon);
 
 		item = gtk_separator_menu_item_new ();
 		gtk_menu_shell_append (GTK_MENU_SHELL (priv->miner_menu), item);
+		gtk_widget_show (item);
 	}
 #endif
 
@@ -854,8 +859,11 @@ status_icon_initialize_miners_menu (TrackerStatusIcon *icon)
 
 	item = gtk_separator_menu_item_new ();
 	gtk_menu_shell_append (GTK_MENU_SHELL (priv->miner_menu), item);
+	gtk_widget_show (item);
 
 	item = gtk_check_menu_item_new_with_mnemonic (_("_Pause All Indexing"));
+	gtk_widget_show (item);
+
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item), FALSE);
 	gtk_menu_shell_append (GTK_MENU_SHELL (priv->miner_menu), item);
 	g_signal_connect (G_OBJECT (item), "toggled",
