@@ -25,10 +25,9 @@
 #include <libtracker-common/tracker-utils.h>
 #include <libtracker-common/tracker-ontologies.h>
 
-#include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-manager.h>
-
 #include <libtracker-data/tracker-data.h>
+#include <libtracker-data/tracker-db-dbus.h>
+#include <libtracker-data/tracker-db-manager.h>
 
 #include "tracker-dbus.h"
 #include "tracker-resources.h"
@@ -43,6 +42,10 @@
 #include "tracker-backup-glue.h"
 #include "tracker-marshal.h"
 
+#ifdef HAVE_DBUS_FD_PASSING
+#include "tracker-steroids.h"
+#endif
+
 static DBusGConnection *connection;
 static DBusGProxy      *gproxy;
 static GSList          *objects;
@@ -206,6 +209,13 @@ tracker_dbus_set_available (gboolean available)
 			                                "NameOwnerChanged",
 			                                G_CALLBACK (name_owner_changed_cb),
 			                                tracker_dbus_get_object (TRACKER_TYPE_RESOURCES));
+
+#ifdef HAVE_DBUS_FD_PASSING
+			dbus_connection_remove_filter (dbus_g_connection_get_connection (connection),
+			                               tracker_steroids_connection_filter,
+			                               tracker_dbus_get_object (TRACKER_TYPE_STEROIDS));
+#endif
+
 			g_slist_foreach (objects, (GFunc) g_object_unref, NULL);
 			g_slist_free (objects);
 			objects = NULL;
@@ -289,6 +299,21 @@ tracker_dbus_register_objects (void)
 	                      TRACKER_RESOURCES_PATH);
 	objects = g_slist_prepend (objects, object);
 
+#ifdef HAVE_DBUS_FD_PASSING
+	/* Add org.freedesktop.Tracker1.Steroids */
+	object = tracker_steroids_new ();
+	if (!object) {
+		g_critical ("Could not create TrackerSteroids object to register");
+		return FALSE;
+	}
+
+	dbus_connection_add_filter (dbus_g_connection_get_connection (connection),
+	                            tracker_steroids_connection_filter,
+	                            object,
+	                            NULL);
+	objects = g_slist_prepend (objects, object);
+#endif
+
 	/* Reverse list since we added objects at the top each time */
 	objects = g_slist_reverse (objects);
 
