@@ -21,16 +21,20 @@
 
 #include "config.h"
 
+#define _GNU_SOURCE
+#include <sched.h>
+#include <pthread.h>
+
 #include <unistd.h>
 #include <sys/types.h>
 
 #include <libtracker-common/tracker-dbus.h>
-#include <libtracker-db/tracker-db-dbus.h>
-#include <libtracker-db/tracker-db-interface-sqlite.h>
 
 #include <libtracker-data/tracker-data-manager.h>
 #include <libtracker-data/tracker-data-update.h>
 #include <libtracker-data/tracker-data-query.h>
+#include <libtracker-data/tracker-db-dbus.h>
+#include <libtracker-data/tracker-db-interface-sqlite.h>
 #include <libtracker-data/tracker-sparql-query.h>
 
 #include "tracker-store.h"
@@ -70,10 +74,10 @@ typedef struct {
 	TrackerStoreTaskType  type;
 	union {
 		struct {
-			gchar       *query;
-			GThread     *running_thread;
-			GTimer      *timer;
-			gpointer     thread_data;
+			gchar        *query;
+			GCancellable *cancellable;
+			GTimer       *timer;
+			gpointer      thread_data;
 		} query;
 		struct {
 			gchar        *query;
@@ -104,6 +108,11 @@ typedef struct {
 
 static GStaticPrivate private_key = G_STATIC_PRIVATE_INIT;
 
+#ifdef __USE_GNU
+/* cpu used for mainloop thread and main update/query thread */
+static int main_cpu;
+#endif /* __USE_GNU */
+
 static void start_handler (TrackerStorePrivate *private);
 
 static void
@@ -129,6 +138,9 @@ store_task_free (TrackerStoreTask *task)
 		if (task->data.query.timer) {
 			g_timer_destroy (task->data.query.timer);
 		}
+		if (task->data.query.cancellable) {
+			g_object_unref (task->data.query.cancellable);
+		}
 	} else {
 		g_free (task->data.update.query);
 	}
@@ -300,15 +312,15 @@ watchdog_cb (gpointer user_data)
 
 	while (running) {
 		TrackerStoreTask *task;
-		GThread *thread;
+		GCancellable *cancellable;
 
 		task = running->data;
 		running = running->next;
-		thread = task->data.query.running_thread;
+		cancellable = task->data.query.cancellable;
 
-		if (thread && private->max_task_time &&
+		if (cancellable && private->max_task_time &&
 		    g_timer_elapsed (task->data.query.timer, NULL) > private->max_task_time) {
-			tracker_data_manager_interrupt_thread (task->data.query.running_thread);
+			g_cancellable_cancel (cancellable);
 		}
 	}
 
@@ -344,6 +356,10 @@ task_finish_cb (gpointer data)
 	task = data;
 
 	if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
+		if (!task->error) {
+			g_cancellable_set_error_if_cancelled (task->data.query.cancellable, &task->error);
+		}
+
 		if (task->callback.query.query_callback) {
 			task->callback.query.query_callback (task->data.query.thread_data, task->error, task->user_data);
 		}
@@ -352,6 +368,7 @@ task_finish_cb (gpointer data)
 			g_clear_error (&task->error);
 		}
 
+		task->data.query.cancellable = g_cancellable_new ();
 		private->running_tasks = g_slist_remove (private->running_tasks, task);
 		check_running_tasks_watchdog (private);
 		private->n_queries_running--;
@@ -445,22 +462,31 @@ pool_dispatch_cb (gpointer data,
 	TrackerStorePrivate *private;
 	TrackerStoreTask *task;
 
+#ifdef __USE_GNU
+	/* special task, only ever sent to main pool */
+	if (GPOINTER_TO_INT (data) == 1) {
+		cpu_set_t cpuset;
+		CPU_ZERO (&cpuset);
+		CPU_SET (main_cpu, &cpuset);
+
+		/* avoid cpu hopping which can lead to significantly worse performance */
+		pthread_setaffinity_np (pthread_self (), sizeof (cpu_set_t), &cpuset);
+		return;
+	}
+#endif /* __USE_GNU */
+
 	private = user_data;
 	task = data;
 
 	if (task->type == TRACKER_STORE_TASK_TYPE_QUERY) {
 		TrackerDBCursor *cursor;
-		GThread *running_thread = g_thread_self ();
 
-		tracker_data_manager_interrupt_thread_reset (running_thread);
-		task->data.query.running_thread = running_thread;
 		cursor = tracker_data_query_sparql_cursor (task->data.query.query, &task->error);
 
-		task->data.query.thread_data = task->callback.query.in_thread (cursor, task->error, task->user_data);
+		task->data.query.thread_data = task->callback.query.in_thread (cursor, task->data.query.cancellable, task->error, task->user_data);
 
 		if (cursor)
 			g_object_unref (cursor);
-		task->data.query.running_thread = NULL;
 
 	} else if (task->type == TRACKER_STORE_TASK_TYPE_UPDATE) {
 		if (task->data.update.batch) {
@@ -596,13 +622,15 @@ queue_idle_destroy (gpointer user_data)
 	private->have_handler = FALSE;
 }
 
-
 void
 tracker_store_init (void)
 {
 	TrackerStorePrivate *private;
 	gint i;
 	const char *tmp;
+#ifdef __USE_GNU
+	cpu_set_t cpuset;
+#endif /* __USE_GNU */
 
 	private = g_new0 (TrackerStorePrivate, 1);
 
@@ -629,10 +657,21 @@ tracker_store_init (void)
 	g_thread_pool_set_max_idle_time (15 * 1000);
 	g_thread_pool_set_max_unused_threads (2);
 
+#ifdef __USE_GNU
+	main_cpu = sched_getcpu ();
+	CPU_ZERO (&cpuset);
+	CPU_SET (main_cpu, &cpuset);
+
+	/* avoid cpu hopping which can lead to significantly worse performance */
+	pthread_setaffinity_np (pthread_self (), sizeof (cpu_set_t), &cpuset);
+	/* lock main update/query thread to same cpu to improve overall performance
+	   main loop thread is essentially idle during query execution */
+	g_thread_pool_push (private->main_pool, GINT_TO_POINTER (1), NULL);
+#endif /* __USE_GNU */
+
 	g_static_private_set (&private_key,
 	                      private,
 	                      private_free);
-
 }
 
 void
@@ -847,9 +886,9 @@ tracker_store_unreg_batches (const gchar *client_id)
 
 		task = running->data;
 
-		if (task->data.query.running_thread &&
-                    g_strcmp0 (task->client_id, client_id) == 0) {
-			tracker_data_manager_interrupt_thread (task->data.query.running_thread);
+		if (task->data.query.cancellable &&
+		    g_strcmp0 (task->client_id, client_id) == 0) {
+			g_cancellable_cancel (task->data.query.cancellable);
 		}
 	}
 
