@@ -25,17 +25,23 @@
 #include <stdio.h>
 
 #include <libtracker-common/tracker-common.h>
-#include <libtracker-db/tracker-db-interface.h>
+#include <libtracker-data/tracker-db-interface.h>
 
 G_BEGIN_DECLS
 
 typedef enum {
 	TRACKER_STORE_PRIORITY_HIGH,
 	TRACKER_STORE_PRIORITY_LOW,
+	TRACKER_STORE_PRIORITY_TURTLE,
 	TRACKER_STORE_N_PRIORITIES
 } TrackerStorePriority;
 
-typedef void (* TrackerStoreSparqlQueryCallback)       (TrackerDBResultSet *result_set,
+typedef void (* TrackerStoreSparqlQueryCallback)       (gpointer         data,
+                                                        GError          *error,
+                                                        gpointer         user_data);
+typedef gpointer
+             (* TrackerStoreSparqlQueryInThread)       (TrackerDBCursor *cursor,
+                                                        GCancellable    *cancellable,
                                                         GError          *error,
                                                         gpointer         user_data);
 typedef void (* TrackerStoreSparqlUpdateCallback)      (GError          *error,
@@ -55,6 +61,7 @@ void         tracker_store_queue_commit           (TrackerStoreCommitCallback ca
                                                    GDestroyNotify destroy);
 void         tracker_store_sparql_query           (const gchar   *sparql,
                                                    TrackerStorePriority priority,
+                                                   TrackerStoreSparqlQueryInThread in_thread,
                                                    TrackerStoreSparqlQueryCallback callback,
                                                    const gchar   *client_id,
                                                    gpointer       user_data,
